% ch6example.m source for the design example in Chapter 6

% Copyright Anderson and Padgett 2009
% Last modified 7/5/09 WTP - original

testovr = 1; % check for overflow when running

[b,a]=ellip(4,1,20,.2); % create an example filter
sosm = tf2sos(b,a);     % convert it to second order sections, dump the 
                        % gain into the first section b's
                        
b1 = sosm(1,1:3);
b2 = sosm(2,1:3);
a1 = sosm(1,4:6);
a2 = sosm(2,4:6);

H1norm = max(abs(freqz(1,a1,1024)))
H2norm = max(abs(freqz(b1,a1,1024)))
s1 = 1/max(H1norm,H2norm)
H3norm = max(abs(s1*freqz(b1,conv(a1,a2),1024)))
H4norm = max(abs(s1*freqz(conv(b1,b2),conv(a1,a2),1024)))
s2 = 1/max(H3norm,H4norm)
s3 = 1/(s1*s2)


b1s = b1*s2;  % adjust scaling as Matlab does
s2s = 1;
s3a = 2/abs(b2(2));
s3b = s3/s3a;
b2s = b2*s3a;

sosmadjusted = [b1s a1; b2s a2]
svadjusted = [s1; s2s; s3b]

%check
Hd = dfilt.df2sos(sosm);
scale(Hd,'Linf','sosreorder','up');
scv = Hd.scalevalues
sosmat = Hd.sosmatrix

%noise power
b1 = sosmat(1,1:3);
b2 = sosmat(2,1:3);
a1 = sosmat(1,4:6);
a2 = sosmat(2,4:6);
s1 = scv(1);
s2 = scv(2);
s3 = scv(3);

N = 2000;
temp = zeros(N,1);
g1 = impz(conv(b1,b2),conv(a1,a2),N)*s2*s3;
g2 = impz(b2,a2,N)*s3;
g3 = temp; g3(1) = s3;
g4 = temp; g4(1) = 1;
sige = 2^(-30)/12;

sig1 = sige*g1'*g1
sig2 = sige*g2'*g2
sig3 = sige*g3'*g3
sig4 = sige*g4'*g4

sigout = sige*(g1'*g1 + g2'*g2 + g3'*g3 + g4'*g4) % sum of square sums

% power spectra
G1b = conv(b1,a2)*s2*s3;
G1a = conv(a1,a2);
G2b = b2*s3;
G2a = a2;
G3b = s3;
G3a = 1;
G4b = 1;
G4a = 1;

% Note definition of spectrum: total power = (1/(2*pi))*int(P(w),-pi,pi)
% but only shown from 0 to pi (in dB)
ww = 0:pi/256:pi;
P1 = 10*log10(abs(freqz(G1b,G1a,ww)).^2*sige);
P2 = 10*log10(abs(freqz(G2b,G2a,ww)).^2*sige);
P3 = 10*log10(abs(freqz(G3b,G3a,ww)).^2*sige);
P4 = 10*log10(abs(freqz(G4b,G4a,ww)).^2*sige);
Pout = 10*log10((abs(freqz(G1b,G1a,ww)).^2 + abs(freqz(G2b,G2a,ww)).^2 + ...
    abs(freqz(G3b,G3a,ww)).^2 + abs(freqz(G4b,G4a,ww)).^2)*sige);


figure(1)
plot(ww,P1); title('Power Spectrum at Output Due to Source 1')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
figure(2)
plot(ww,P2); title('Power Spectrum at Output Due to Source 2')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
figure(3)
plot(ww,P3); title('Power Spectrum at Output Due to Source 3')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
figure(4)
plot(ww,P4); title('Power Spectrum at Output Due to Source 4')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
figure(5)
plot(ww,Pout,'Linewidth',2); 
title('Total Power Spectrum at Output Due to All Sources','Fontweight','bold','Fontsize',14)
xlabel('\omega (rad/sample)','Fontweight','bold','Fontsize',14)
ylabel('Power Spectrum (dB)','Fontweight','bold','Fontsize',14)
set(gca,'Fontweight','bold','Fontsize',14);

axis([0 pi -90 -70]);
set(gca,'YTick',[-90:4:-70]);

figure(6)
subplot(221)
plot(ww,P1); title('Power Spectrum at Output Due to Source 1')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
subplot(222)
plot(ww,P2); title('Power Spectrum at Output Due to Source 2')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
subplot(223)
plot(ww,P3); title('Power Spectrum at Output Due to Source 3')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
subplot(224)
plot(ww,P4); title('Power Spectrum at Output Due to Source 4')
xlabel('\omega (rad/sample)')
ylabel('Power Spectrum (dB)')
pos6 = get(gcf,'position');
set(gcf,'position',[pos6(1), pos6(2), 800, 520]);  % make it bigger so labels don't overlap


doprint = 1;
if doprint,
    if 0, % don't do this
        figure(1)
        %print -dpdf ch5spectrum1.pdf  % produces full page
        print -depsc ch5spectrum1.eps
        % produces "fit to figure" size
        !epstopdf ch5spectrum1.eps
        figure(2)
        print -depsc ch5spectrum2.eps
        !epstopdf ch5spectrum2.eps
        figure(3)
        print -depsc ch5spectrum3.eps
        !epstopdf ch5spectrum3.eps
        figure(4)
        print -depsc ch5spectrum4.eps
        !epstopdf ch5spectrum4.eps
    end % if 0
    figure(5)
    print -depsc ch5spectrum5.eps
    !epstopdf ch5spectrum5.eps
    figure(6)
    print -depsc ch5spectrum1234.eps
    !epstopdf ch5spectrum1234.eps
end  %if doprint

% SNR calculations
speech = wavread('began.wav');
sp_out = filter(Hd,speech);
sp_pwr = sum(sp_out.^2) / length(sp_out) % average power, not total
noise_pwr = sigout % as in sigma^2_out
SNR = sp_pwr/noise_pwr
SNRdB = 10*log10(SNR)

% check SNR with simulation
% note slight differences expected since coefficients are quantized here
% but not above
Q15 = numerictype(1,16,15); % Q15 = s16,15
xxq = fi(speech,Q15);
Hd = setq3(Hd,[16 15 32 31 16 14 16 14]);  
Hd.ScaleValueFracLength = 11;
coeffcheck(Hd);  % my own function to watch for coefficient problems 
        [fixout, floatout, rerr, snr, ovrerr, guardout] = ...
            filterboth(Hd,xxq,xxq,testovr);
sp_pwr_sim = sum(double(fixout).^2)/ length(fixout)
noise_pwr_sim = sum(rerr.^2)/ length(fixout)
SNRsim = sp_pwr_sim/noise_pwr_sim
SNRdBsim = 10*log10(SNRsim)
snr_filterboth = snr

% SNR in freq domain
figure(7)
Sxx = pwelch(sqrt(2*pi)*sp_out,1024,512,ww,'twosided'); % signal spectrum
Sout = 10*log10(abs(Sxx));
plot(ww,Sout,'-',ww,Pout,'-.','Linewidth',2); 
axis([0 pi -100 10]);
set(gca,'YTick',[-100:20:20]);
title('Output Signal and Noise Spectra','Fontweight','bold','Fontsize',14)
xlabel('\omega (radians/sample)','Fontweight','bold','Fontsize',14)
ylabel('Power Spectrum (dB)','Fontweight','bold','Fontsize',14)
legend('Signal', 'Noise')
set(gca,'Fontweight','bold','Fontsize',14);

doprint7 = 1;
if doprint7
    print -depsc spectra.eps
    !epstopdf spectra.eps
end  %if doprint

% SNR in passband
Iwpb = find(ww<0.2*pi); % only look in passband
Spb = 10.^(Sout(Iwpb)/10); % careful of units, can't add dB
Npb = 10.^(Pout(Iwpb)/10);
SNRpb = 10*log10(sum(Spb)/sum(Npb))








