% quantization_error_audio.m demo

% Copyright Anderson and Padgett 2009
% Last modified 7/5/09 WTP - original

[speech,fs] = wavread('the pipe began to leak 2.wav');

disp('The speech file was recorded at 16 bit resolution.')
disp('You may requantize it to any resolution you choose between 15 and 2 bits.')
disp('You should at least try 4, 6, 8 and 10 bits.')
disp(' ')
bits = input('How many bits would you like to use? ');
if bits<2 | bits>15
    error('bits input out of range')
end
disp(' ')
disp('You will now hear the original signal, then the quantized signal, ')
disp('and finally the error signal (normalized so it is always audible).')
disp(' ')

% Assuming the 16 bit signal is stored in the wav file as integers 
% between 2^15 and -2^15, and then scaled back to a range of 1 to -1 
% (much as a Q1.15 number would be treated), we can re-quantize
% by scaling to a new range and rounding to integers before scaling 
% back to the +/- 1 range.  The number of bits is reduced by one to 
% accomodate the natural range of 2^15 in a 16 bit signal.
intspeech = round(2^(bits-1)*speech);
qspeech = intspeech/2^(bits-1);

% write some wav files for those without Matlab
qfilename = ['qspeech_' num2str(bits) 'bits.wav'];
err = qspeech-speech;
errfilename = ['norm_errspeech_' num2str(bits) 'bits.wav'];
warning off % avoid clipping warnings due to unrepresentable values near 1
wavwrite(qspeech,fs,qfilename)
wavwrite(0.25*err/max(abs(err)),fs,errfilename)
warning on

range = 11300:11499;
figure(1)
subplot(311)
plot(speech(range)); title('Original Speech')
subplot(312)
plot(qspeech(range)); title('Quantized Speech')
subplot(313)
plot(err(range)); title('Quantization Error')

sound(speech,fs)
sound(qspeech,fs)
sound(0.25*err/max(abs(err)),fs)

disp('Note that while the quantized speech remains (barely) intelligible')
disp('at any bit resolution, the signal can be distinguished in the noise at')
disp('both very high and very low bit resolutions.  This is because at low')
disp('resolutions the quantization steps are large compared to the signal')
disp('(violating our assumptions), and at high resolutions there are very')
disp('few total error levels (preventing the error from being approximately')
disp('uniform).')