function [maxpercenterr, err1, err2] = coeffcheck(Hd);
% function [maxpercenterr, coefferr, scaleerr] = coeffcheck(Hd);(for df2sos)
% function [maxpercenterr, numerr, denerr] = coeffcheck(Hd);    (for df2)
% function to test coefficients and scale values for overflow or underflow.
% only works for df2 and df2sos right now
% pass a dfilt object, get back the largest percent error in the
% coefficients or scale values

% Copyright 2008
% Wayne T. Padgett
% Last Modified - 
% wtp 9/30/04 ece497
% wtp 4/5/07 added df2 support
% wtp 1/30/08 added df1sos support

if Hd.arithmetic ~= 'fixed'
    warning('no need to check floating point coefficients')
    
elseif strcmp(Hd.filterstructure,'Direct-Form II, Second-Order Sections')
    Href = reffilter(Hd);

    sosf = Href.sosmatrix;
    scf = Href.scalevalues;

    sos = Hd.sosmatrix;
    sc = Hd.scalevalues;

    coefferr = 100*(sosf-sos)./sosf;
    scaleerr = 100*(scf-sc)./scf;
    maxpercenterr = max([max(max(abs(coefferr))),max(scaleerr)]);
    err1 = coefferr;
    err2 = scaleerr;

    if maxpercenterr > 0.1,
        warning('Better check your coefficients and scale values for overflow or underflow')
        sosf
        sos
        scf
        sc
    end

elseif strcmp(Hd.filterstructure,'Direct-Form I, Second-Order Sections')
    Href = reffilter(Hd);

    sosf = Href.sosmatrix;
    scf = Href.scalevalues;

    sos = Hd.sosmatrix;
    sc = Hd.scalevalues;

    coefferr = 100*(sosf-sos)./sosf;
    scaleerr = 100*(scf-sc)./scf;
    maxpercenterr = max([max(max(abs(coefferr))),max(scaleerr)]);
    err1 = coefferr;
    err2 = scaleerr;

    if maxpercenterr > 0.1,
        warning('Better check your coefficients and scale values for overflow or underflow')
        sosf
        sos
        scf
        sc
    end

elseif strcmp(Hd.filterstructure,'Direct-Form II')
    Href = reffilter(Hd);

    numf = Href.numerator;
    denf = Href.denominator;

    num = Hd.numerator;
    den = Hd.denominator;

    numerr = 100*(numf-num)./numf;
    denerr = 100*(denf-den)./denf;
    maxpercenterr = max([max(max(abs(numerr))),max(denerr)]);
    err1 = numerr;
    err2 = denerr;

    if maxpercenterr > 0.1,
        warning('Better check your coefficients for overflow or underflow')
        numf
        num
        denf
        den
    end

else
    error('I didn''t write this for anything but df2, df2sos, and df1sos (yet)');
end

