function Hd = setq3(Hd,quantization);
% function Hd = setq3(Hd,quantization);
% function to set a filter object''s quantization bit lengths.
% quantization should be a vector with values
% [WordlengthA FraclengthA WordlengthB FracLengthB WordlengthC FracLengthC WordlengthD FracLengthD]
% where for dffir
% A specifies Coefficients, Inputs, Outputs
% and B specifies Products and Accumulators
%
% and for df2 or df2sos
% A specifies Inputs, Outputs
% and B specifies Products and Accumulators
% and C specifies Denominator Coefficients
% and D specifies Numerator Coefficients

% feel free to create your own custom version for your needs, but please
% name it something slightly different.

% Copyright 2008
% Wayne T. Padgett
% Last Modified - 3/22/08 WTP - fixed df1 stateformat
% Last Modified - 1/30/08 WTP - adding df1 support
%                 4/11/07 WTP - added stateformat parameters to DF2 DF2SOS
% Last Modified - 9/15/05 WTP - fixed df2 problem (no stages)



Hd.arithmetic = 'fixed';
specifyall(Hd); % eliminate all automatic scaling of values (not very practical)

Wa = quantization(1);
Fa = quantization(2);
Wb = quantization(3);
Fb = quantization(4);

if (strcmp(lower(Hd.filterstructure),'direct-form fir'))

    Hd.CoeffWordLength = Wa;
    Hd.NumFracLength = Fa;

    Hd.InputWordLength = Wa;
    Hd.InputFracLength = Fa;

    Hd.OutputWordLength = Wa;
    Hd.OutputFracLength = Fa;

    Hd.ProductWordLength = Wb;
    Hd.ProductFracLength = Fb;

    Hd.AccumWordLength = Wb;
    Hd.AccumFracLength = Fb;

elseif (strcmp(lower(Hd.filterstructure),'direct-form ii') | strcmp(lower(Hd.filterstructure),'direct-form ii, second-order sections'))

    Wc = quantization(5);
    Fc = quantization(6);
    Wd = quantization(7);
    Fd = quantization(8);

    Hd.CoeffWordLength = Wa;
    Hd.NumFracLength = Fd;
    Hd.DenFracLength = Fc; % So Wc must be Wa or there's a problem
    if (Wa ~= Wc)
        warning('Wa and Wc must be the same');
    end
    
    if (strcmp(lower(Hd.filterstructure),'direct-form ii, second-order sections'))
        Hd.StageInputWordLength = Wa;
        Hd.StageInputFracLength = Fa;
        Hd.StageOutputWordLength = Wa;
        Hd.StageOutputFracLength = Fa;
    end
    
    Hd.InputWordLength = Wa;
    Hd.InputFracLength = Fa;

    Hd.statewordlength = Wa;
    Hd.statefraclength = Fa;

    Hd.OutputWordLength = Wa;
    Hd.OutputFracLength = Fa;

    Hd.ProductWordLength = Wb;
    Hd.NumProdFracLength = Fb;
    Hd.DenProdFracLength = Fb;

    Hd.AccumWordLength = Wb;
    Hd.NumAccumFracLength = Fb;
    Hd.DenAccumFracLength = Fb;

elseif (strcmp(lower(Hd.filterstructure),'direct-form i') | strcmp(lower(Hd.filterstructure),'direct-form i, second-order sections'))
    Wc = quantization(5);
    Fc = quantization(6);
    Wd = quantization(7);
    Fd = quantization(8);

    Hd.CoeffWordLength = Wa;
    Hd.NumFracLength = Fd;
    Hd.DenFracLength = Fc; % So Wc must be Wa or there's a problem
    if (Wa ~= Wc)
        warning('Wa and Wc must be the same');
    end
    
    Hd.InputWordLength = Wa;
    Hd.InputFracLength = Fa;

    Hd.numstatewordlength = Wa;
    Hd.numstatefraclength = Fa;
    Hd.denstatewordlength = Wa;
    Hd.denstatefraclength = Fa;

    Hd.OutputWordLength = Wa;
    Hd.OutputFracLength = Fa;

    Hd.ProductWordLength = Wb;
    Hd.NumProdFracLength = Fb;
    Hd.DenProdFracLength = Fb;

    Hd.AccumWordLength = Wb;
    Hd.NumAccumFracLength = Fb;
    Hd.DenAccumFracLength = Fb;

else
    error('Filter type unknown')
end



